/**
// Quicklists - Nautilus quicklist
// Copyright 2012  Damian Brun
//
// Licence: GPLv2+
**/

const PopupMenu = imports.ui.popupMenu;
const Shell = imports.gi.Shell;

const St = imports.gi.St;
const GLib = imports.gi.GLib;
const Gio = imports.gi.Gio;

const DEFAULT_DIRS = [
    GLib.UserDirectory.DIRECTORY_DOCUMENTS,
    GLib.UserDirectory.DIRECTORY_MUSIC,
    GLib.UserDirectory.DIRECTORY_PICTURES,
    GLib.UserDirectory.DIRECTORY_DOWNLOAD,
    GLib.UserDirectory.DIRECTORY_VIDEOS,
];

function append (popUpMenu, icon, iconSize, iconSymbolic, addDefaultPlaces, addBookmarks, addMounts) {
    let pos = 0;

    function _add_place_item(name, path, _icon) {
        let item = new PopupMenu.PopupMenuItem(name);
        item.connect('activate', function() {
            try { GLib.spawn_command_line_async('nautilus '+ path); }
            catch (e) { global.logError('' + e); }
        });
        if (_icon)
            item.addActor(_icon, { align: St.Align.END });
        popUpMenu.addMenuItem(item, pos);
        pos++;
    }

    function _add_separator() {
        let item = new PopupMenu.PopupSeparatorMenuItem();
        popUpMenu.addMenuItem(item, pos);
        pos++;
    }

    function _get_icon(gfile) {
        let icon = null;
        let gicon = null
        try {
            let info = gfile.query_info(iconSymbolic ? 'standard::symbolic-icon' : 'standard::icon', 0, null);
        if (iconSymbolic)
            gicon = info.get_symbolic_icon();
        else
            gicon = info.get_icon();
        } catch (e if e instanceof Gio.IOErrorEnum) {
            let _name = iconSymbolic ? 'folder-symbolic' : 'folder';
            gicon = new Gio.ThemedIcon({ name: _name});
        }
        if (gicon)
            icon = new St.Icon({ gicon: gicon, icon_size: iconSize });
        return icon;
    }

    let homePath = GLib.get_home_dir();
    if (addDefaultPlaces) 
        _add_place_item("Home", homePath, icon ? _get_icon(Gio.File.new_for_path(homePath)) : null);

    let defaultDirs = [];
    for (let i = 0; i < DEFAULT_DIRS.length; i++) {
        let specialPath = GLib.get_user_special_dir(DEFAULT_DIRS[i]);
        if (specialPath == homePath)
            continue;
        defaultDirs.push(Gio.File.new_for_path(specialPath));
        if (addDefaultPlaces)
            _add_place_item(GLib.path_get_basename(specialPath), specialPath,
                            icon ? _get_icon(defaultDirs[i]) : null);
    }

    if (addBookmarks) {
        let bookmarksFilePath = GLib.build_filenamev([GLib.get_user_config_dir(), 'gtk-3.0', 'bookmarks']);;
        if (!GLib.file_test(bookmarksFilePath, GLib.FileTest.EXISTS))
            bookmarksFilePath = GLib.build_filenamev([GLib.get_home_dir(), '.gtk-bookmarks']);
        if (GLib.file_test(bookmarksFilePath, GLib.FileTest.EXISTS)) {
            let content = Shell.get_file_contents_utf8_sync(bookmarksFilePath);
            /* file://<path/to/folder><space><label><\n> */
            let lines = content.split('\n');
            for (let i = 0; i < lines.length; i++) {
                let line = lines[i].split(' ');
                let uri = line[0];
                if (!uri)
                    continue;

                let gfile = Gio.File.new_for_uri(uri);
                if (!gfile.query_exists(null) && gfile.is_native())
                    continue;

                let f = false;
                for (let i = 0; i < defaultDirs.length; i++) {
                    if (gfile.equal(defaultDirs[i])) {
                        f = true;
                        break;
                    }
                }
                if (f)
                    continue;

                let label = null;
                if (line.length > 1)
                    label = line.slice(1).join(' ');
                else
                    label = GLib.path_get_basename(uri);

                _add_place_item(label, uri, icon ? _get_icon(gfile) : null);
            }
        }
    }

    if (pos > 0)
        _add_separator();
}

